package com.sun.xml.internal.txw2.output;

import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.ContentHandler;
import java.io.IOException;
import com.sun.xml.internal.txw2.TxwException;
import java.io.FileOutputStream;
import javax.xml.transform.stream.StreamResult;
import java.io.Writer;
import java.io.UnsupportedEncodingException;
import java.io.OutputStream;

public class StreamSerializer implements XmlSerializer
{
    private final SaxSerializer serializer;
    private final XMLWriter writer;
    
    public StreamSerializer(OutputStream out) {
        this(createWriter(out));
    }
    
    public StreamSerializer(OutputStream out, String encoding) throws UnsupportedEncodingException {
        this(createWriter(out, encoding));
    }
    
    public StreamSerializer(Writer out) {
        this(new StreamResult(out));
    }
    
    public StreamSerializer(StreamResult streamResult) {
        final OutputStream[] autoClose = { null };
        if (streamResult.getWriter() != null) {
            this.writer = createWriter(streamResult.getWriter());
        }
        else if (streamResult.getOutputStream() != null) {
            this.writer = createWriter(streamResult.getOutputStream());
        }
        else if (streamResult.getSystemId() != null) {
            String fileURL = streamResult.getSystemId();
            fileURL = this.convertURL(fileURL);
            try {
                FileOutputStream fos = new FileOutputStream(fileURL);
                autoClose[0] = fos;
                this.writer = createWriter(fos);
            }
            catch (IOException e) {
                throw new TxwException(e);
            }
        }
        else {
            throw new IllegalArgumentException();
        }
        this.serializer = new SaxSerializer(this.writer, this.writer, false) {
            @Override
            public void endDocument() {
                super.endDocument();
                if (autoClose[0] != null) {
                    try {
                        autoClose[0].close();
                    }
                    catch (IOException e) {
                        throw new TxwException(e);
                    }
                    autoClose[0] = null;
                }
            }
        };
    }
    
    private StreamSerializer(XMLWriter writer) {
        this.writer = writer;
        this.serializer = new SaxSerializer(writer, writer, false);
    }
    
    private String convertURL(String url) {
        url = url.replace('\\', '/');
        url = url.replaceAll("//", "/");
        url = url.replaceAll("//", "/");
        if (url.startsWith("file:/")) {
            if (url.substring(6).indexOf(":") > 0) {
                url = url.substring(6);
            }
            else {
                url = url.substring(5);
            }
        }
        return url;
    }
    
    @Override
    public void startDocument() {
        this.serializer.startDocument();
    }
    
    @Override
    public void beginStartTag(String uri, String localName, String prefix) {
        this.serializer.beginStartTag(uri, localName, prefix);
    }
    
    @Override
    public void writeAttribute(String uri, String localName, String prefix, StringBuilder value) {
        this.serializer.writeAttribute(uri, localName, prefix, value);
    }
    
    @Override
    public void writeXmlns(String prefix, String uri) {
        this.serializer.writeXmlns(prefix, uri);
    }
    
    @Override
    public void endStartTag(String uri, String localName, String prefix) {
        this.serializer.endStartTag(uri, localName, prefix);
    }
    
    @Override
    public void endTag() {
        this.serializer.endTag();
    }
    
    @Override
    public void text(StringBuilder text) {
        this.serializer.text(text);
    }
    
    @Override
    public void cdata(StringBuilder text) {
        this.serializer.cdata(text);
    }
    
    @Override
    public void comment(StringBuilder comment) {
        this.serializer.comment(comment);
    }
    
    @Override
    public void endDocument() {
        this.serializer.endDocument();
    }
    
    @Override
    public void flush() {
        this.serializer.flush();
        try {
            this.writer.flush();
        }
        catch (IOException e) {
            throw new TxwException(e);
        }
    }
    
    private static XMLWriter createWriter(Writer w) {
        DataWriter dw = new DataWriter(new BufferedWriter(w));
        dw.setIndentStep("  ");
        return dw;
    }
    
    private static XMLWriter createWriter(OutputStream os, String encoding) throws UnsupportedEncodingException {
        XMLWriter writer = createWriter(new OutputStreamWriter(os, encoding));
        writer.setEncoding(encoding);
        return writer;
    }
    
    private static XMLWriter createWriter(OutputStream os) {
        try {
            return createWriter(os, "UTF-8");
        }
        catch (UnsupportedEncodingException e) {
            throw new Error(e);
        }
    }
}
